using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;
using NUnit.Framework;

namespace TopCoder.Rules.DependencyMap.View.AccuracyTests
{
	/// <summary>
	/// Builds different maps and test the layout.
	/// 
    /// @author cucu
    /// Copyright (c)2005, TopCoder, Inc. All rights reserved
    /// </summary>
    [TestFixture]
	public class FixedGridLayoutManagerAccuracyTests
	{
        private DependencyRenderer renderer = null;
        private Dependency a = null;
        private Dependency b = null;
        private Dependency c = null;
        private Dependency d = null;
        private Dependency e = null;
        private Dependency f = null;
        private Dependency g = null;

        [SetUp]
        protected void SetUp()
        {

            renderer = new DependencyRenderer();

            a = new Dependency("A");
            b = new Dependency("B");
            c = new Dependency("C");
            d = new Dependency("D");
            e = new Dependency("E");
            f = new Dependency("F");
            g = new Dependency("G");
        }
        /// <summary>
        /// Test overriding a shape.
        /// </summary>
        [Test]
        public void TestShapeOverride()
        {
			renderer.AddDiagramRenderer(new SVGRenderer(@"../../test_files/accuracytests/TestShapeOverride.svg"));
			a = new Dependency("A", ShapeFactory.SeenElement);
            renderer.Render(a);
        }

        /// <summary>
        /// Test overriding the default padding and margin.
        /// </summary>
        [Test]
        public void TestPaddingAndMarginAndSize()
        {
			renderer.AddDiagramRenderer(new SVGRenderer(@"../../test_files/accuracytests/TestPaddingAndMargin.svg"));
			renderer.LayoutManager = new FixedGridLayoutManager(37, 49, 17);
			String text = "this is a Padding and Margin test";
			a = new Dependency(text);
            renderer.Render(a);
        }
        
        /// <summary>
        /// Test with only one dependency.
        /// </summary>
        [Test]
        public void TestOneDependency()
        {
			renderer.AddDiagramRenderer(new SVGRenderer(@"../../test_files/accuracytests/TestOneDependency.svg"));
			renderer.Render(a);
        }

        /// <summary>
        /// Test A that depends on B
        ///    A
        ///    |
        ///    B
        /// </summary>
        [Test]
        public void TestOneChild()
        {
			renderer.AddDiagramRenderer(new SVGRenderer(@"../../test_files/accuracytests/TestOneChild.svg"));
			a.AddDependant(b);
            renderer.Render(a);
        }

        /// <summary>
        /// Test A that depends on B and C
        ///    A
        ///   / \
        ///  B   C
        /// </summary>
        [Test]
        public void TestTwoChild()
        {
			renderer.AddDiagramRenderer(new SVGRenderer(@"../../test_files/accuracytests/TestTwoChild.svg"));
			a.AddDependant(b);
            a.AddDependant(c);
            renderer.Render(a);
        }

        /// <summary>
        /// Test A that depends on B, C and D
        ///    A
        ///   /|\
        ///  B C D
        /// </summary>
        [Test]
        public void TestTreeChild()
        {
			renderer.AddDiagramRenderer(new SVGRenderer(@"../../test_files/accuracytests/TestTreeChild.svg"));
			a.AddDependant(b);
            a.AddDependant(c);
            a.AddDependant(d);
            renderer.Render(a);
        }

        /// <summary>
        /// Test a chain of 4 dependencies.
        ///    A
        ///    |
        ///    B
        ///    |
        ///    C
        ///    |
        ///    D
        /// </summary>
        [Test]
        public void TestChain()
        {
			renderer.AddDiagramRenderer(new SVGRenderer(@"../../test_files/accuracytests/TestChain.svg"));
			a.AddDependant(b);
            b.AddDependant(c);
            c.AddDependant(d);
            renderer.Render(a);
        }
        /// <summary>
        /// Test the dependencies as the figure 2 of the specification:
        ///      A
        ///    /   \
        ///    B    E 
        ///   / \
        ///  C   D
        /// </summary>
        [Test]
        public void TestSpec1()
        {
			renderer.AddDiagramRenderer(new SVGRenderer(@"../../test_files/accuracytests/TestSpec1.svg"));
			a.AddDependant(b);
            a.AddDependant(e);
            b.AddDependant(c);
            b.AddDependant(d);
            e.AddDependant(f);
            renderer.Render(a);
        }
        
        /// <summary>
        /// Test the dependencies as the figure 6 of the specification:
        ///      A
        ///    /   \
        ///    B    C 
        ///  // \
        /// D E  F
        ///   |  /\
        ///   F G C
        /// </summary>
        [Test]
        public void TestSpec2()
        {
			renderer.AddDiagramRenderer(new SVGRenderer(@"../../test_files/accuracytests/TestSpec2.svg"));
			a.AddDependant(b);
            a.AddDependant(c);
            b.AddDependant(d);
            b.AddDependant(e);
            b.AddDependant(f);
            e.AddDependant(f);
            f.AddDependant(g);
            f.AddDependant(c);
            renderer.Render(a);
        }

	}
}
