package com.topcoder.rules.dependencymap.view;

import java.awt.image.BufferedImage;

/**
 * The BufferedImage Renderer will implement the DiagramRenderer and will render
 * diagrams to a buffered image. Please note that the buffered image is
 * recreated on every render. The application should call getBufferedImage()
 * after the diagram has been rendered to save a reference to the rendered
 * diagram.
 */
public class BufferedImageRenderer implements DiagramRenderer {

	/**
	 * This variable represents the width of the buffered image to be created.
	 * This variable is set in the constructor, is mutable by the setter and
	 * will never be <= 0. This variable is referenced in the getter and
	 * getGraphics methods.
	 */
	private int width;

	/**
	 * This variable represents the height of the buffered image to be created.
	 * This variable is set in the constructor, is mutable by the setter and
	 * will never be <= 0. This variable is referenced in the getter and
	 * getGraphics methods.
	 */
	private int height;

	/**
	 * This variable represents the image type of the buffered image to be
	 * created. This variable is set in the constructor, is mutable by the
	 * setter and will can be any BufferedImage.TYPExxx value . This variable is
	 * referenced in the getter and getGraphics methods.
	 */
	private int imageType;

	/**
	 * This variable represents the <b>last </b> buffered image that was drawn
	 * to and is recreated each time the getGraphics method is called. This
	 * value is set in the getGraphics method, is mutatable and can be null if
	 * the renderer has not been rendered to yet. The variable is referenced in
	 * the getter method.
	 */
	private BufferedImage bufferedImage = null;

	/**
	 * Constructs the BufferedImageRenderer from the given width/height/image
	 * type. This method simply save the args to the like named variables.
	 * 
	 * 
	 * 
	 * @param width
	 *            the width of the image to create
	 * @param height
	 *            the height of the image to create
	 * @param imageType
	 *            the image type (BufferedImage.TYPExxx) of the image to use
	 * @throws IllegalArgumentException if width is <= 0 or height is <= 0 or 
	 * 		   imageType isn't one of the defined types in BufferedImage
	 */
	public BufferedImageRenderer(int width, int height, int imageType) {
		if (width <= 0) {
			throw new IllegalArgumentException("width shoud be positive");
		}
		if (height <= 0) {
			throw new IllegalArgumentException("height shoud be positive");
		}
		
		this.width = width;
		this.height = height;
		this.imageType = imageType;
	}

	/** lock-begin */

	/**
	 * This is the getter method for the BufferedImage. This method simply
	 * returns the bufferedImage reference
	 * 
	 * 
	 * 
	 * @return The buffered image that was last drawn or null if none have been
	 *         drawn yet
	 */
	public BufferedImage getBufferedImage() {
		return this.bufferedImage;
	}

	/** lock-begin */

	/**
	 * This is the getter method for the imageType. This method simply returns
	 * the imageType
	 * 
	 * 
	 * 
	 * @return The image type - will be one of the BufferedImage.TYPExxx
	 */
	public int getImageType() {		
		return imageType;
	}

	/**
	 * The setter for the imageType. This method simply saves the value to the
	 * imageType
	 * 
	 * 
	 * 
	 * @param imageType
	 *            the image type (BufferedImage.TYPExxx) of the image to use
	 * @throws IllegalArgumentValue
	 *             if imageType isn't one of the defined types in BufferedImage
	 */
	public void setImageType(int imageType) {
		this.imageType = imageType;
	}

	/**
	 * This is the getter method for the width. This method simply returns the
	 * width
	 * 
	 * 
	 * 
	 * @return the width of the image being created
	 */
	public int getWidth() {
		return width;
	}

	/**
	 * The setter for the width. This method simply saves the value to the width
	 * 
	 * 
	 * 
	 * @param width
	 *            the height of the image to create
	 * @throws IllegalArgumentException
	 *             if width is <= 0
	 */
	public void setWidth(int width) {
		this.width = width;
	}

	/**
	 * This is the getter method for the height. This method simply returns the
	 * height
	 * 
	 * 
	 * 
	 * @return the height of the image
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * The setter for the height. This method simply saves the value to the
	 * height
	 * 
	 * 
	 * 
	 * @param height
	 *            the height of the image to create
	 * @throws IllegalArgumentException
	 *             if height is <= 0
	 */
	public void setHeight(int height) {
		this.height = height;
	}

	/**
	 * This method will: 1) Create the bufferedImage using the
	 * width/height/imageType and save it to the variable. 2) Create and return
	 * a Graphics2DAdapter using the created bufferedImage.createGraphics()
	 * 
	 * 
	 * 
	 * @param dependency
	 *            The dependency that will be rendered
	 * @param width unused
	 * @param height unused
	 * @return A graphics object to render with
	 * @throws NullPointerException
	 *             if dependency is null
	 */
	public RendererGraphics getGraphics(Dependency dependency, int width, int height) {
		this.bufferedImage = new BufferedImage(this.width, this.height, this.imageType);
		return new Graphics2DAdapter(this.bufferedImage.createGraphics());
	}
}