/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view;

import junit.framework.TestCase;
import java.io.File;
import java.awt.geom.GeneralPath;

/**
 * Demonstration for the Component.
 *
 * Demo is split into two parts. The first part runs roughly the code supplied by the designer.
 * The second part builds complex dependency map as in the algorithm description.
 *
 * Notice the output of the demo are not cleared on purpose. Client can view the files with
 * an SVG explorer.
 *
 * @author TCSDeveloper
 */
public class ComponentDemo extends TestCase {

    /**
     * Get an oval shape from application.
     *
     * @return an oval shape
     */
    private GeneralPath getOvalShape() {
        GeneralPath shape = new GeneralPath();
        shape.moveTo(20, 0);
        shape.quadTo(37, 2, 39, 20);
        shape.quadTo(37, 37, 20, 39);
        shape.quadTo(2, 37, 0, 20);
        shape.quadTo(2, 2, 20, 0);
        shape.closePath();
        return shape;
    }

    /**
     * Get a start shape from application.
     *
     * @return a star shape.
     */
    private GeneralPath getStarShape() {
        GeneralPath shape = new GeneralPath();
        shape.moveTo(20, 0);
        shape.lineTo(27, 12);
        shape.lineTo(39, 20);
        shape.lineTo(27, 27);
        shape.lineTo(20, 39);
        shape.lineTo(12, 27);
        shape.lineTo(0, 20);
        shape.lineTo(12, 12);
        shape.closePath();
        return shape;
    }

    /**
     * The first demo. Runs the code supplied by the designer from CS.
     *
     * @throws RendererException to JUnit.
     */
    public void testComponentSpecificationDemo() throws RendererException {
        // Create the shape factory
        ShapeFactory shapeFactory = new ShapeFactory();

        // Override a common seen shape with an oval (application provided method)
        shapeFactory.setShape(ShapeFactory.SEEN_ELEMENT, getOvalShape());

        // Add a new shape (application provided method)
        int BEING_BUILT = 4;
        shapeFactory.setShape(BEING_BUILT, getStarShape());

        // Create the dependencies
        Dependency configManager = new Dependency("Config Manager");

        Dependency spellCheck = new Dependency("Spell Check", BEING_BUILT);
        spellCheck.addDependant(configManager);

        Dependency phoneticPatternMatching = new Dependency("Phonetic Pattern Matching");

        Dependency phoneticSpellCheck = new Dependency("Phonetic Spell Check");
        phoneticSpellCheck.addDependant(configManager);
        phoneticSpellCheck.addDependant(spellCheck);
        phoneticSpellCheck.addDependant(phoneticPatternMatching);

        // Create the Dependency Renderer and add some diagram renderers
        DependencyRenderer renderer = new DependencyRenderer(shapeFactory);
        renderer.addDiagramRenderer(new SVGRenderer(new File("test_files")));

        // Render some of the dependencies
        renderer.render(phoneticSpellCheck);
        renderer.render(spellCheck);
    }

    /**
     * The second demo. Builds complex dependency map as in the algorithm description.
     *
     * @throws RendererException to JUnit.
     */
    public void testComplexDependencyRendering() throws RendererException {
        // Create the dependencies
        Dependency a = new Dependency("A");
        Dependency b = new Dependency("B");
        Dependency c = new Dependency("C");
        Dependency d = new Dependency("D");
        Dependency e = new Dependency("E");
        Dependency f = new Dependency("F");
        Dependency g = new Dependency("G");

        // Build the mapping
        a.addDependant(b);
        a.addDependant(c);
        b.addDependant(d);
        b.addDependant(e);
        b.addDependant(f);
        e.addDependant(f);
        f.addDependant(g);
        f.addDependant(c);

        // Create the Dependency Renderer and add some diagram renderers
        DependencyRenderer renderer = new DependencyRenderer();
        renderer.addDiagramRenderer(new SVGRenderer(new File("test_files/Complex Dependency Map.svg")));

        // Render the first dependency
        renderer.render(a);
    }

}
