/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view;

import junit.framework.TestCase;

/**
 * Unit test cases for Dependency.
 *
 * Testing of Dependency focuses on its various manipulation methods.
 *
 * @author TCSDeveloper
 */
public class DependencyUnitTests extends TestCase {

    /**
     * The title to use.
     */
    private String title = "title";

    /**
     * The shapeOverride to use.
     */
    private int shapeOverride = 3;

    /**
     * The dependency instance to test.
     */
    private Dependency dependency = null;

    /**
     * A dependant to use.
     */
    private Dependency dependant1 = new Dependency("a");

    /**
     * Another dependant to use.
     */
    private Dependency dependant2 = new Dependency("b");

    /**
     * Create a new dependency for each test.
     */
    protected void setUp() {
        dependency = new Dependency(title, shapeOverride);
    }

    /**
     * Create instance with title.
     */
    public void testCreate_Title() {
        dependency = new Dependency(title);
        assertEquals("Title mismatches.", title, dependency.getTitle());
    }

    /**
     * Create instance with null title.
     */
    public void testCreate_NullTitle() {
        try {
            new Dependency(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Create instance with empty title.
     */
    public void testCreate_EmptyTitle() {
        try {
            new Dependency(" ");
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with title and shapeOverride.
     */
    public void testCreate_TitleShapeOverride() {
        dependency = new Dependency(title, shapeOverride);
        assertEquals("Title mismatches.", title, dependency.getTitle());
        assertEquals("ShapeOverride mismatches.", shapeOverride, dependency.getShapeOverride());
    }

    /**
     * Create instance with null title.
     */
    public void testCreate_NullTitle2() {
        try {
            new Dependency(null, shapeOverride);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Create instance with empty title.
     */
    public void testCreate_EmptyTitle2() {
        try {
            new Dependency(" ", shapeOverride);
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with invalid shapeOverride.
     */
    public void testCreate_InvalidShapeOverride() {
        try {
            new Dependency(title, 0);
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Test getTitle().
     */
    public void testGetTitle() {
        assertEquals("Title mismatches.", title, dependency.getTitle());
    }

    /**
     * Test getShapeOverride().
     */
    public void testGetShapeOverride() {
        assertEquals("ShapeOverride mismatches.", shapeOverride, dependency.getShapeOverride());
    }

    /**
     * Test addDependant().
     */
    public void testAddDependant() {
        dependency.addDependant(dependant1);
        assertEquals("Incorrect size.", 1, dependency.getDependants().size());
        assertTrue("Does not contain the dependant.", dependency.getDependants().contains(dependant1));

        dependency.addDependant(dependant2);
        assertEquals("Incorrect size.", 2, dependency.getDependants().size());
        assertTrue("Does not contain the dependant.", dependency.getDependants().contains(dependant1));
        assertTrue("Does not contain the dependant.", dependency.getDependants().contains(dependant2));
    }

    /**
     * Test addDependant() with null dependant.
     */
    public void testAddDependant_NullDependant() {
        try {
            dependency.addDependant(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test removeDependant().
     */
    public void testRemoveDependant() {
        dependency.addDependant(dependant1);
        dependency.addDependant(dependant2);

        assertTrue("Did not remove the dependant.", dependency.removeDependant(dependant1));
        assertFalse("Removed the dependant again.", dependency.removeDependant(dependant1));
        assertEquals("Incorrect size.", 1, dependency.getDependants().size());
        assertFalse("Still contains the dependant.", dependency.getDependants().contains(dependant1));

        assertTrue("Did not remove the dependant.", dependency.removeDependant(dependant2));
        assertFalse("Removed the dependant again.", dependency.removeDependant(dependant2));
        assertEquals("Incorrect size.", 0, dependency.getDependants().size());
        assertFalse("Still contains the dependant.", dependency.getDependants().contains(dependant2));
    }

    /**
     * Test removeDependant() with null dependant.
     */
    public void testRemoveDependant_NullDependant() {
        try {
            dependency.removeDependant(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test getDependants().
     */
    public void testGetDependants() {
        dependency.addDependant(dependant1);
        dependency.addDependant(dependant2);

        assertEquals("Incorrect size.", 2, dependency.getDependants().size());
        assertTrue("Does not contain the dependant.", dependency.getDependants().contains(dependant1));
        assertTrue("Does not contain the dependant.", dependency.getDependants().contains(dependant2));
    }

}
